@@ -1,16 +1,28 @@
-use std::collections::HashMap;
+//! Abstract syntax tree for WADL documents.
 use iri_string::spec::IriSpec;
-use iri_string::types::{RiReferenceString};
+use iri_string::types::RiReferenceString;
+use std::collections::HashMap;
 use url::Url;
 
+/// Identifier for a resource, method, parameter, etc.
 pub type Id = String;
 
+/// Parameter style
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub enum ParamStyle {
+    /// Specifies a component of the representation formatted as a string encoding of the parameter value according to the rules of the media type.
     Plain,
+
+    /// Specifies a matrix URI component.
     Matrix,
+
+    /// Specifies a URI query parameter represented according to the rules for the query component media type specified by the queryType attribute.
     Query,
+
+    /// Specifies a HTTP header that pertains to the HTTP request (resource or request) or HTTP response (response)
     Header,
+
+    /// The parameter is represented as a string encoding of the parameter value and is substituted into the value of the path attribute of the resource element as described in section 2.6.1.
     Template,
 }
 
@@ -20,21 +32,26 @@ pub struct Application {
     /// Resources defined at the application level.
     pub resources: Vec<Resources>,
 
+    /// Resource types defined at the application level.
     pub resource_types: Vec<ResourceType>,
 
     /// Documentation for the application.
     pub docs: Vec<Doc>,
 
+    /// List of grammars
     pub grammars: Vec<Grammar>,
 
+    /// Representations defined at the application level.
     pub representations: Vec<RepresentationDef>,
 }
 
 impl Application {
+    /// Get a resource type by its ID.
     pub fn get_resource_type_by_id(&self, id: &str) -> Option<&ResourceType> {
         self.resource_types.iter().find(|rt| id == rt.id.as_str())
     }
 
+    /// Get a resource type by its href, which may be a fragment or a full URL.
     pub fn get_resource_type_by_href(&self, href: &Url) -> Option<&ResourceType> {
         // TODO(jelmer): Check that href matches us?
         if let Some(fragment) = href.fragment() {
@@ -45,7 +62,7 @@ impl Application {
     }
 
     /// Iterate over all resources defined in this application.
-    pub fn iter_resources(&self) -> impl Iterator<Item=(Url, &Resource)> {
+    pub fn iter_resources(&self) -> impl Iterator<Item = (Url, &Resource)> {
         self.resources
             .iter()
             .flat_map(|rs| rs.resources.iter().map(|r| (r.url(rs.base.as_ref()), r)))
@@ -59,7 +76,7 @@ impl Application {
     }
 
     /// Iterate over all types defined in this application.
-    pub fn iter_referenced_types(&self) -> impl Iterator<Item=String> + '_ {
+    pub fn iter_referenced_types(&self) -> impl Iterator<Item = String> + '_ {
         self.iter_resources()
             .flat_map(|(_u, r)| r.iter_referenced_types())
             .chain(
@@ -70,7 +87,7 @@ impl Application {
     }
 
     /// Iterate over all parameters defined in this application.
-    pub fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
+    pub fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
         self.iter_resources()
             .flat_map(|(_u, r)| r.iter_all_params())
             .chain(
@@ -81,7 +98,8 @@ impl Application {
             .chain(
                 self.representations
                     .iter()
-                    .flat_map(|r| r.iter_all_params()))
+                    .flat_map(|r| r.iter_all_params()),
+            )
     }
 }
 
@@ -94,6 +112,7 @@ impl std::str::FromStr for Application {
 }
 
 #[derive(Debug)]
+/// A collection of resources.
 pub struct Resources {
     /// The base URL for the resources.
     pub base: Option<Url>,
@@ -103,14 +122,22 @@ pub struct Resources {
 }
 
 #[derive(Debug)]
+/// A grammar
 pub struct Grammar {
+    /// The href of the grammar.
     pub href: RiReferenceString<IriSpec>,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
+/// A reference to a resource type.
 pub enum ResourceTypeRef {
+    /// A reference to a resource type defined in the same document.
     Id(Id),
+
+    /// A reference to a resource type defined in another document.
     Link(Url),
+
+    /// An empty reference.
     Empty,
 }
 
@@ -124,7 +151,9 @@ impl std::str::FromStr for ResourceTypeRef {
                 if let Some(s) = s.strip_prefix('#') {
                     Ok(ResourceTypeRef::Id(s.to_string()))
                 } else {
-                    Ok(ResourceTypeRef::Link(s.parse().map_err(|e| format!("{}", e))?))
+                    Ok(ResourceTypeRef::Link(
+                        s.parse().map_err(|e| format!("{}", e))?,
+                    ))
                 }
             }
         }
@@ -133,16 +162,21 @@ impl std::str::FromStr for ResourceTypeRef {
 
 #[test]
 fn parse_resource_type_ref() {
-    use std::str::FromStr;
     use crate::ast::ResourceTypeRef::*;
+    use std::str::FromStr;
     assert_eq!(Empty, ResourceTypeRef::from_str("").unwrap());
-    assert_eq!(Id("id".to_owned()), ResourceTypeRef::from_str("#id").unwrap());
-    assert_eq!(Link(Url::parse("https://example.com").unwrap()),
-               ResourceTypeRef::from_str("https://example.com").unwrap());
-    
+    assert_eq!(
+        Id("id".to_owned()),
+        ResourceTypeRef::from_str("#id").unwrap()
+    );
+    assert_eq!(
+        Link(Url::parse("https://example.com").unwrap()),
+        ResourceTypeRef::from_str("https://example.com").unwrap()
+    );
 }
 
 impl ResourceTypeRef {
+    /// Return the ID of the resource type reference.
     pub fn id(&self) -> Option<&str> {
         match self {
             ResourceTypeRef::Id(id) => Some(id),
@@ -153,6 +187,7 @@ impl ResourceTypeRef {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
+/// An option element defines one of a set of possible values for the parameter represented by its parent param element.
 pub struct Options(HashMap<String, Option<mime::Mime>>);
 
 impl std::hash::Hash for Options {
@@ -167,30 +202,37 @@ impl std::hash::Hash for Options {
 }
 
 impl Options {
+    /// Create a new options object
     pub fn new() -> Self {
         Self(HashMap::new())
     }
 
+    /// Number of items in this Options
     pub fn len(&self) -> usize {
         self.0.len()
     }
 
-    pub fn iter(&self) -> impl Iterator<Item=(&str, Option<&mime::Mime>)> {
+    /// Iterate over all items in this Options
+    pub fn iter(&self) -> impl Iterator<Item = (&str, Option<&mime::Mime>)> {
         self.0.iter().map(|(k, v)| (k.as_str(), v.as_ref()))
     }
 
-    pub fn keys(&self) -> impl Iterator<Item=&str> {
+    /// Return an iterator over all keys
+    pub fn keys(&self) -> impl Iterator<Item = &str> {
         self.0.keys().map(|k| k.as_str())
     }
 
+    /// Check if this Options is empty
     pub fn is_empty(&self) -> bool {
         self.0.is_empty()
     }
 
+    /// Insert a new key-value pair into this Options
     pub fn insert(&mut self, key: String, value: Option<mime::Mime>) {
         self.0.insert(key, value);
     }
 
+    /// Get the value for a key
     pub fn get(&self, key: &str) -> Option<&Option<mime::Mime>> {
         self.0.get(key)
     }
@@ -209,6 +251,7 @@ impl From<Vec<&str>> for Options {
 }
 
 #[derive(Debug, Clone)]
+/// A resource
 pub struct Resource {
     /// The ID of the resource.
     pub id: Option<Id>,
@@ -246,7 +289,7 @@ impl Resource {
     }
 
     /// Iterate over all parameters defined in this resource.
-    pub(crate) fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
+    pub(crate) fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
         let mut params = self.params.iter().collect::<Vec<_>>();
 
         params.extend(self.subresources.iter().flat_map(|r| r.iter_all_params()));
@@ -256,7 +299,7 @@ impl Resource {
     }
 
     /// Iterate over all types referenced by this resource.
-    pub fn iter_referenced_types(&self) -> impl Iterator<Item=String> + '_ {
+    pub fn iter_referenced_types(&self) -> impl Iterator<Item = String> + '_ {
         self.iter_all_params().map(|p| p.r#type.clone())
     }
 }
@@ -284,22 +327,34 @@ fn test_resource_url() {
 }
 
 #[derive(Debug, Clone)]
+/// A HTTP Method
 pub struct Method {
+    /// Identifier of this method
     pub id: Id,
+
+    /// The name of the method.
     pub name: String,
+
+    /// The docs for the method.
     pub docs: Vec<Doc>,
+
+    /// The request for the method.
     pub request: Request,
+
+    /// The responses for the method.
     pub responses: Vec<Response>,
 }
 
 impl Method {
-    fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
-        self.request.iter_all_params().chain(
-            self.responses.iter().flat_map(|r| r.iter_all_params()))
+    fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
+        self.request
+            .iter_all_params()
+            .chain(self.responses.iter().flat_map(|r| r.iter_all_params()))
     }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Default)]
+/// Documentation
 pub struct Doc {
     /// The title of the documentation.
     pub title: Option<String>,
@@ -315,6 +370,7 @@ pub struct Doc {
 }
 
 impl Doc {
+    /// Create a new documentation object.
     pub fn new(content: String) -> Self {
         Self {
             content,
@@ -324,7 +380,9 @@ impl Doc {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
+/// A link to another resource.
 pub struct Link {
+    /// The resource type of the link.
     pub resource_type: Option<ResourceTypeRef>,
 
     /// Optional token that identifies the relationship of the resource identified by the link to
@@ -338,49 +396,87 @@ pub struct Link {
     /// of the ancestor representation element's profile attribute.
     pub reverse_relation: Option<String>,
 
+    /// Optional documentation
     pub doc: Option<Doc>,
 }
 
-
 #[derive(Debug, Clone, PartialEq, Eq)]
+/// A parameter
 pub struct Param {
+    /// The style of the parameter.
     pub style: ParamStyle,
+
+    /// The ID of the parameter.
     pub id: Option<Id>,
+
+    /// The name of the parameter.
     pub name: String,
+
+    /// The type of the parameter.
     pub r#type: String,
+
+    /// Path of the parameter.
     pub path: Option<String>,
+
+    /// Whether the parameter is required.
     pub required: bool,
+
+    /// Whether the parameter is repeating.
     pub repeating: bool,
+
+    /// The fixed value of the parameter.
     pub fixed: Option<String>,
+
+    /// The documentation for the parameter.
     pub doc: Option<Doc>,
+
+    /// The links for the parameter.
     pub links: Vec<Link>,
+
+    /// The options for the parameter.
     pub options: Option<Options>,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Default)]
+/// A representation definition
 pub struct RepresentationDef {
+    /// The ID of the representation.
     pub id: Option<Id>,
+
+    /// The media type of the representation.
     pub media_type: Option<mime::Mime>,
+
+    /// The element of the representation.
     pub element: Option<String>,
+
+    /// The profile of the representation.
     pub profile: Option<String>,
+
+    /// The documentation for the representation.
     pub docs: Vec<Doc>,
+
+    /// The parameters for the representation.
     pub params: Vec<Param>,
 }
 
 impl RepresentationDef {
-    fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
+    fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
         self.params.iter()
     }
 }
 
 #[derive(Debug, Clone)]
+/// A reference to a representation.
 pub enum RepresentationRef {
     /// A reference to a representation defined in the same document.
     Id(Id),
+
+    /// A reference to a representation defined in another document.
     Link(Url),
 }
 
 impl RepresentationRef {
+    /// Return the ID of the representation reference.
     pub fn id(&self) -> Option<&str> {
         match self {
             RepresentationRef::Id(id) => Some(id),
@@ -390,12 +486,17 @@ impl RepresentationRef {
 }
 
 #[derive(Debug, Clone)]
+/// A representation
 pub enum Representation {
+    /// A reference to a representation defined in the same document.
     Reference(RepresentationRef),
+
+    /// A definition of a representation.
     Definition(RepresentationDef),
 }
 
 impl Representation {
+    /// Return the content type of this representation.
     pub fn media_type(&self) -> Option<&mime::Mime> {
         match self {
             Representation::Reference(_) => None,
@@ -403,6 +504,7 @@ impl Representation {
         }
     }
 
+    /// Return the URL of this representation.
     pub fn url(&self, base_url: &Url) -> Option<Url> {
         match self {
             Representation::Reference(RepresentationRef::Id(id)) => {
@@ -415,6 +517,7 @@ impl Representation {
         }
     }
 
+    /// Return the definition of this representation.
     pub fn as_def(&self) -> Option<&RepresentationDef> {
         match self {
             Representation::Reference(_) => None,
@@ -422,13 +525,13 @@ impl Representation {
         }
     }
 
-    pub fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
+    /// Iterate over all parameters defined in this representation.
+    pub fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
         // TODO: Make this into a proper iterator
-        let params =
-            match self {
-                Representation::Reference(_) => vec![],
-                Representation::Definition(d) => d.iter_all_params().collect::<Vec<_>>(),
-            };
+        let params = match self {
+            Representation::Reference(_) => vec![],
+            Representation::Definition(d) => d.iter_all_params().collect::<Vec<_>>(),
+        };
 
         params.into_iter()
     }
@@ -438,14 +541,25 @@ impl Representation {
 fn test_representation_url() {
     let base_url = Url::parse("http://example.com").unwrap();
     let r = Representation::Reference(RepresentationRef::Id("foo".to_string()));
-    assert_eq!(r.url(&base_url).unwrap(), Url::parse("http://example.com#foo").unwrap());
-    let r = Representation::Reference(RepresentationRef::Link(Url::parse("http://example.com#foo").unwrap()));
-    assert_eq!(r.url(&base_url).unwrap(), Url::parse("http://example.com#foo").unwrap());
+    assert_eq!(
+        r.url(&base_url).unwrap(),
+        Url::parse("http://example.com#foo").unwrap()
+    );
+    let r = Representation::Reference(RepresentationRef::Link(
+        Url::parse("http://example.com#foo").unwrap(),
+    ));
+    assert_eq!(
+        r.url(&base_url).unwrap(),
+        Url::parse("http://example.com#foo").unwrap()
+    );
     let r = Representation::Definition(RepresentationDef {
         id: Some("foo".to_string()),
         ..Default::default()
     });
-    assert_eq!(r.url(&base_url).unwrap(), Url::parse("http://example.com#foo").unwrap());
+    assert_eq!(
+        r.url(&base_url).unwrap(),
+        Url::parse("http://example.com#foo").unwrap()
+    );
 }
 
 #[test]
@@ -460,6 +574,7 @@ fn test_representation_id() {
 }
 
 impl RepresentationDef {
+    /// Fully qualify the URL of this representation.
     pub fn url(&self, base_url: &Url) -> Option<Url> {
         if let Some(id) = &self.id {
             let mut url = base_url.clone();
@@ -472,14 +587,20 @@ impl RepresentationDef {
 }
 
 #[derive(Debug, Default, Clone)]
+/// A request
 pub struct Request {
+    /// The docs for the request.
     pub docs: Vec<Doc>,
+
+    /// The parameters for the request.
     pub params: Vec<Param>,
+
+    /// The representations for the request.
     pub representations: Vec<Representation>,
 }
 
 impl Request {
-    fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
+    fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
         self.params.iter().chain(
             self.representations
                 .iter()
@@ -490,15 +611,23 @@ impl Request {
 }
 
 #[derive(Debug, Clone, Default)]
+/// A response
 pub struct Response {
+    /// The docs for the response.
     pub docs: Vec<Doc>,
+
+    /// The parameters for the response.
     pub params: Vec<Param>,
+
+    /// The status of the response.
     pub status: Option<i32>,
+
+    /// The representations for the response.
     pub representations: Vec<Representation>,
 }
 
 impl Response {
-    fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
+    fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
         self.params.iter().chain(
             self.representations
                 .iter()
@@ -509,24 +638,37 @@ impl Response {
 }
 
 #[derive(Debug)]
+/// A resource type
 pub struct ResourceType {
+    /// The ID of the resource type.
     pub id: Id,
+
+    /// The query type of the resource type.
     pub query_type: mime::Mime,
+
+    /// The methods defined at this level.
     pub methods: Vec<Method>,
+
+    /// The docs for the resource type.
     pub docs: Vec<Doc>,
+
+    /// The subresources of the resource type.
     pub subresources: Vec<Resource>,
+
+    /// The params for the resource type.
     pub params: Vec<Param>,
 }
 
 impl ResourceType {
-    pub(crate) fn iter_all_params(&self) -> impl Iterator<Item=&Param> {
+    /// Iterate over all parameters defined in this resource type.
+    pub(crate) fn iter_all_params(&self) -> impl Iterator<Item = &Param> {
         self.params
             .iter()
             .chain(self.methods.iter().flat_map(|m| m.iter_all_params()))
     }
 
     /// Returns an iterator over all types referenced by this resource type.
-    pub fn iter_referenced_types(&self) -> impl Iterator<Item=String> + '_ {
+    pub fn iter_referenced_types(&self) -> impl Iterator<Item = String> + '_ {
         self.iter_all_params().map(|p| p.r#type.clone())
     }
 }
